import 'dart:async';
import 'dart:convert';
import 'dart:io';

import 'package:flutter/services.dart';

class AppierFlutterPlatform {
  static const String version = '1.1.0-alpha';
  static const MethodChannel _channel = MethodChannel('appier_flutter');

  static final AppierFlutterPlatform _instance = AppierFlutterPlatform();
  static AppierFlutterPlatform get instance => _instance;

  bool _isConfigured = false;

  Future<void> configure(String appId,
      {String? senderId, String? appGroup, bool isDev = false}) async {
    if (appId.trim().isEmpty) {
      print('APPIER: Invalid AppId passed');
      return;
    }

    _isConfigured = true;

    try {
      await _channel.invokeMethod<void>('configure', {
        'appId': appId,
        'senderId': senderId,
        'appGroup': appGroup,
        'isDev': isDev,
        'version': version
      });
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> enableIdfaConsent() async {
    if (!Platform.isIOS) {
      // This is an iOS only API.
      return;
    }
    try {
      await _channel
          .invokeMethod<void>('setIdfaConsent', {'idfaConsent': true});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> disableIdfaConsent() async {
    if (!Platform.isIOS) {
      // This is an iOS only API.
      return;
    }
    try {
      await _channel
          .invokeMethod<void>('setIdfaConsent', {'idfaConsent': false});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setUniversalLinkDomains(List<String> linkDomains) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>(
          'setUniversalLinkDomains', {'linkDomains': linkDomains});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> logEvent(String eventName,
      {Map<String, dynamic>? parameters, double? vts, String? vtsCurr}) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    if (eventName.trim().isEmpty) {
      print('APPIER: Invalid Event Name passed');
      return;
    }

    try {
      await _channel.invokeMethod<void>('logEvent', {
        'eventName': eventName,
        'parameters': parameters,
        'vts': vts,
        'vtsCurr': vtsCurr
      });
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> flush() async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>('flush', <String, dynamic>{});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setUserId(String userId) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>('setUserId', {'userId': userId});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setName(String name) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>('setName', {'name': name});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setFirstName(String firstName) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel
          .invokeMethod<void>('setFirstName', {'firstName': firstName});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setLastName(String lastName) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>('setLastName', {'lastName': lastName});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setCity(String city) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>('setCity', {'city': city});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setEmail(String email) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>('setEmail', {'email': email});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setPhoneNumber(String phoneNo) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>('setPhoneNumber', {'phoneNo': phoneNo});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setDayOfBirth(int day) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>('setDayOfBirth', {'day': day});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setMonthOfBirth(int month) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>('setMonthOfBirth', {'month': month});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setYearOfBirth(int year) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>('setYearOfBirth', {'year': year});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setCustomKey(String key, dynamic value) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    if (key.trim().isEmpty) {
      print('APPIER: Invalid key passed');
      return;
    }

    if (value is! String &&
        value is! double &&
        value is! int &&
        value is! num &&
        value is! bool &&
        value != null) {
      print('APPIER: value must be string, number, boolean or null');
    }

    try {
      await _channel.invokeMethod<void>(
          'setCustomKey', <String, dynamic>{'key': key, 'value': value});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setAttributionWindow(int seconds) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel
          .invokeMethod<void>('setAttributionWindow', {'seconds': seconds});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<void> setClickAttributionWindow(int seconds) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    try {
      await _channel.invokeMethod<void>(
          'setClickAttributionWindow', {'seconds': seconds});
    } on PlatformException catch (error) {
      print(error.message);
    }
  }

  Future<Map<String, dynamic>?> getRecommendation(String scenarioId,
      {String? productId, Map<String, dynamic>? parameters}) async {
    if (!_isConfigured) {
      const errorMsg = 'APPIER: Must configure the SDK first!';
      print(errorMsg);
      return Future.error(errorMsg);
    }

    if (scenarioId == '') {
      const errorMsg = 'APPIER: Empty scenarioId';
      print(errorMsg);
      return Future.error(errorMsg);
    }

    try {
      final json = await _channel.invokeMethod<String>('getRecommendation', {
        'scenarioId': scenarioId,
        'productId': productId,
        'parameters': parameters,
      });
      if (json == null) {
        return null;
      }
      return await jsonDecode(json) as Map<String, dynamic>?;
    } on PlatformException catch (error) {
      print(error.message);
      return null;
    }
  }

  Future<void> logRecommendationClicked(String scenarioId, int modelId,
      String productId, String recommendationId) async {
    if (!_isConfigured) {
      print('APPIER: Must configure the SDK first!');
      return;
    }

    if (scenarioId == '' ||
        modelId <= 0 ||
        productId == '' ||
        recommendationId == '') {
      print('APPIER: String arguments should not be empty strings. '
          'Numeric argument should not be less than or equal to zero.');
      return;
    }

    try {
      await _channel.invokeMethod<void>('logRecommendationClicked', {
        'scenarioId': scenarioId,
        'modelId': modelId,
        'productId': productId,
        'recommendationId': recommendationId,
      });
    } on PlatformException catch (error) {
      print(error.message);
    }
  }
}
