import 'dart:io';
import 'package:appier_flutter/appier_flutter.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';
import 'package:url_launcher/url_launcher.dart';

const String defaultURL = 'https://qa-webside-testing.netlify.app/';

class WebviewWidget extends StatefulWidget {
  const WebviewWidget({Key? key}) : super(key: key);
  @override
  _WebviewWidgetState createState() => _WebviewWidgetState();
}

class _WebviewWidgetState extends State<WebviewWidget> {
  final GlobalKey _webViewKey = GlobalKey();

  InAppWebViewController? _webViewController;
  final InAppWebViewGroupOptions _options = InAppWebViewGroupOptions(
      crossPlatform: InAppWebViewOptions(
        useShouldOverrideUrlLoading: true,
        mediaPlaybackRequiresUserGesture: false,
      ),
      android: AndroidInAppWebViewOptions(
        useHybridComposition: true,
      ),
      ios: IOSInAppWebViewOptions(
        allowsInlineMediaPlayback: true,
      ));

  late PullToRefreshController _pullToRefreshController;
  String _url = '';
  double _progress = 0;
  final _urlController = TextEditingController();

  @override
  void initState() {
    super.initState();

    _pullToRefreshController = PullToRefreshController(
      options: PullToRefreshOptions(
        color: Colors.blue,
      ),
      onRefresh: () async {
        if (Platform.isAndroid) {
          await _webViewController?.reload();
        } else if (Platform.isIOS) {
          await _webViewController?.loadUrl(
              urlRequest: URLRequest(url: await _webViewController?.getUrl()));
        }
      },
    );
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SafeArea(
        child: Column(children: <Widget>[
      TextField(
        decoration: InputDecoration(prefixIcon: Icon(Icons.search)),
        controller: _urlController,
        keyboardType: TextInputType.url,
        onSubmitted: (value) {
          var url = Uri.parse(value);
          if (url.scheme.isEmpty) {
            url = Uri.parse('https://www.google.com/search?q=$value');
          }
          _webViewController?.loadUrl(urlRequest: URLRequest(url: url));
        },
      ),
      Expanded(
        child: Stack(
          children: [
            InAppWebView(
              key: _webViewKey,
              initialUrlRequest: URLRequest(url: Uri.parse(defaultURL)),
              initialOptions: _options,
              pullToRefreshController: _pullToRefreshController,
              onWebViewCreated: (controller) async {
                _webViewController = controller;
                await AppierFlutter.enableWebSdkBridge(
                    inAppWebViewController: controller);
              },
              onLoadStart: (controller, url) async {
                await AppierFlutter.enableWebSdkBridge(
                    inAppWebViewController: controller);
                setState(() {
                  _url = url.toString();
                  _urlController.text = _url;
                });
              },
              androidOnPermissionRequest:
                  (controller, origin, resources) async {
                return PermissionRequestResponse(
                    resources: resources,
                    action: PermissionRequestResponseAction.GRANT);
              },
              shouldOverrideUrlLoading: (controller, navigationAction) async {
                final uri = navigationAction.request.url!;

                if (![
                  'http',
                  'https',
                  'file',
                  'chrome',
                  'data',
                  'javascript',
                  'about'
                ].contains(uri.scheme)) {
                  if (await canLaunch(_url)) {
                    // Launch the App
                    await launch(
                      _url,
                    );
                    // and cancel the request
                    return NavigationActionPolicy.CANCEL;
                  }
                }

                return NavigationActionPolicy.ALLOW;
              },
              onLoadStop: (controller, url) async {
                await _pullToRefreshController.endRefreshing();
                setState(() {
                  _url = url.toString();
                  _urlController.text = _url;
                });
              },
              onLoadError: (controller, url, code, message) {
                _pullToRefreshController.endRefreshing();
              },
              onProgressChanged: (controller, progress) {
                if (progress == 100) {
                  _pullToRefreshController.endRefreshing();
                }
                setState(() {
                  _progress = progress / 100;
                  _urlController.text = _url;
                });
              },
              onUpdateVisitedHistory: (controller, url, androidIsReload) {
                setState(() {
                  _url = _url.toString();
                  _urlController.text = _url;
                });
              },
              onConsoleMessage: (controller, consoleMessage) {
                print(consoleMessage);
              },
            ),
            if (_progress < 1.0)
              LinearProgressIndicator(value: _progress)
            else
              Container(),
          ],
        ),
      ),
      ButtonBar(
        alignment: MainAxisAlignment.center,
        children: <Widget>[
          ElevatedButton(
            onPressed: () {
              _webViewController?.goBack();
            },
            child: Icon(Icons.arrow_back),
          ),
          ElevatedButton(
            onPressed: () {
              _webViewController?.goForward();
            },
            child: Icon(Icons.arrow_forward),
          ),
          ElevatedButton(
            onPressed: () {
              _webViewController?.reload();
            },
            child: Icon(Icons.refresh),
          ),
        ],
      ),
    ]));
  }
}
